// MMConversionDlg.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "MMConversionDlg.h"
#include <math.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MMConversionDlg dialog
typedef enum
{
  NO_IMAGE,
  IMAGE,
  FIRST_POINT
} MODE;


MMConversionDlg::MMConversionDlg(CWnd* pParent /*=NULL*/)
: CDialog(MMConversionDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(MMConversionDlg)
		// NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  CString fname = "";
  m_root_color.SetColor(RGB(255,0,0));
  m_pixels_per_mm = 1.0;
  m_point1 = CPoint(-1,-1);
  m_point2 = CPoint(-1,-1);
  m_mode = NO_IMAGE;
}


void MMConversionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MMConversionDlg)
	DDX_Control(pDX, IDC_LINE_COLOR, m_root_color);
	DDX_Control(pDX, IDC_TEXT_STATIC, m_text);
	DDX_Control(pDX, IDC_IMAGE, m_image);
	DDX_Control(pDX, IDC_EDIT_PIXELS, m_edit_pixels);
	DDX_Control(pDX, IDC_EDIT_CONVERSION, m_edit_conversion);
	DDX_Control(pDX, IDC_EDIT_MILLI, m_edit_milli);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MMConversionDlg, CDialog)
	//{{AFX_MSG_MAP(MMConversionDlg)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_PAINT()
	ON_EN_KILLFOCUS(IDC_EDIT_MILLI, OnKillfocusEditMilli)
	ON_BN_CLICKED(IDC_PICTURE, OnPicture)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MMConversionDlg message handlers
BOOL MMConversionDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
  
  // initial setup
  CString str;
  str.Format("%5.1f", m_pixels_per_mm);  
  m_edit_conversion.SetWindowText(str);
  m_edit_milli.EnableWindow(false);
  m_cur_pen = AfxGetApp()->LoadCursor(IDC_CUR_PEN);  

// maximize window
//  WINDOWPLACEMENT wp;
//  wp.showCmd = SW_SHOWMAXIMIZED;
//  SetWindowPlacement(&wp);

  return TRUE;  
}


void MMConversionDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
  Redraw(dc);
}

// redraws the line along with the picture
void MMConversionDlg::Redraw(CDC &dc)
{
  CRgn clip_rgn;
  CRect rect;
  m_image.GetWindowRect(&rect);
  ScreenToClient(rect);
  
  // finds the top left point of the image  
  CPoint pt = rect.CenterPoint();
  pt.x -= m_img.Width() / 2;
  pt.y -= m_img.Height() / 2;
  
  
  // creates the clip region
  clip_rgn.CreateRectRgnIndirect(CRect(pt.x,pt.y,pt.x+m_img.Width(),pt.y+m_img.Height()));
  dc.SelectClipRgn(&clip_rgn);
  
  // hides directions if image isn't loaded
  if(m_mode != NO_IMAGE) m_text.ShowWindow(SW_SHOW);
  else m_text.ShowWindow(SW_HIDE);
  
  // draw image
  blepo::Draw(m_img, dc, pt.x, pt.y);

  // draw line
  if (m_point1.x >= 0)
  {
    CPen pen(PS_SOLID, 1, m_root_color.GetColor());
    CPen* old = dc.SelectObject(&pen);
    dc.MoveTo(m_point1);
    dc.LineTo(m_point2);
    dc.SelectObject(old);
  }
}


void MMConversionDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
  CClientDC dc(this);
  CRect rect;
  m_image.GetWindowRect(&rect);
  ScreenToClient(rect);
  
  // finds the top left point of the image
  CPoint pt = rect.CenterPoint();
  pt.x -= m_img.Width() / 2;
  pt.y -= m_img.Height() / 2;
  
  // creates the rect around the image
  rect = CRect(pt.x,pt.y,pt.x+m_img.Width(),pt.y+m_img.Height());

  if(m_mode != NO_IMAGE)
  {
    if (rect.PtInRect(point))
    {
      SetCursor(m_cur_pen);
      if (m_mode == FIRST_POINT)  
      {
        m_point2 = point;
        Redraw(dc);
      }
    }
  }
  
  CDialog::OnMouseMove(nFlags, point);
}

// grabs the beginning and ending points of the line
void MMConversionDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
  CRect rect;
  m_image.GetWindowRect(&rect);
  ScreenToClient(rect);

  // finds the top left point of the image
  CPoint pt = rect.CenterPoint();
  pt.x -= m_img.Width() / 2;
  pt.y -= m_img.Height() / 2;
  
  // creates the rect around the image
  rect = CRect(pt.x,pt.y,pt.x+m_img.Width(),pt.y+m_img.Height());

  if (rect.PtInRect(point) && m_mode != NO_IMAGE)
  {
    // this is the beginning point
    if (m_mode != FIRST_POINT)
    {
      m_point1 = point;
      m_point2 = point;
      m_mode = FIRST_POINT;
    }
    else
    {
      // this is the second point. length of line is calculated
      float dx = (float) m_point1.x -  (float) m_point2.x;
      float dy = (float) m_point1.y - (float) m_point2.y;
      float pixels = (float) (sqrt(dx * dx + dy * dy) + 0.5f);
      float milli = (float) (pixels / m_pixels_per_mm);
      CString str;
      str.Format("%5.0f", pixels);
      m_edit_pixels.SetWindowText(str);
      str.Format("%5.3f", milli);
      m_edit_milli.SetWindowText(str);
      Recalculate();
      m_mode = IMAGE;
    }
  }
	CDialog::OnLButtonDown(nFlags, point);
}

void MMConversionDlg::OnKillfocusEditMilli() 
{
  Recalculate();	
}

void MMConversionDlg::OnOK() 
{
  if (GetFocus() == GetDlgItem(IDC_EDIT_MILLI))
  {
    Recalculate();
    return;
  }	
	CDialog::OnOK();
}

// loads the picture onto the dialog
void MMConversionDlg::OnPicture() 
{
//  CRect rect;
//  CRgn clip_rgn;
  CClientDC dc(this);

  m_edit_milli.EnableWindow();
  CString str;
  str.Format("%5.1f", 1.0);  
  m_edit_milli.SetWindowText(str);
  

  CFileDialog dlg(TRUE, "jpg", NULL, OFN_HIDEREADONLY, "JPEG Files (*.jpg)|*.jpg|Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*|");  
  dlg.m_ofn.lpstrTitle = "Load calibration image";

  int ret = dlg.DoModal();
  if (ret == IDOK)
  {
    fname = dlg.GetFileName();
    bool success = blepo::Load(fname, &m_img);
    if (!success)
    {
      CString str;
      str.Format("Unable to load file '%s'", (const char*) fname);
      AfxMessageBox(str);
    } else
    {
      m_mode = IMAGE;
      Redraw(dc);
    }
  }
}

// calculates the pixel to mm conversion
void MMConversionDlg::Recalculate()
{
  CString str;
  m_edit_milli.GetWindowText(str);
  float mm = (float) atof(str);
  str.Format("%5.1f", mm);
  m_edit_milli.SetWindowText(str);
  m_edit_pixels.GetWindowText(str);
  float pixels = (float) atof(str);
  m_pixels_per_mm = pixels / mm;
  str.Format("%5.1f", m_pixels_per_mm);
  m_edit_conversion.SetWindowText(str);
}

void MMConversionDlg::SetLineColor(COLORREF color)
{
  m_root_color.SetColor(color);
}

void MMConversionDlg::SetConversion(float conversion)
{
  m_pixels_per_mm = conversion;
}
